import tomlkit
from xdg_base_dirs import xdg_config_home

# This code implements a singleton-like pattern.
# 
# The global variable _raw_config contains the cached config loaded
# from a TOML config file.
#
# _load_config populates the variable if it has not already been loaded.
#
# Functions that need the config, can use _load_config() which will
# always return a copy of the config regardless of it needed to be loaded
# or not.
#
# Eventually, this approach could also enable a live-reload function.
# 
# A command could be introduced that would reset _raw_config to None,
# and the next read would reload the fresh data.

_raw_config = None

def _load_config():
    global _raw_config
    if not _raw_config:
        # ~/.config/tt/tt.toml
        with open(xdg_config_home() / "tt/tt.toml", "r") as f:
            _raw_config = tomlkit.load(f)
    return _raw_config


def get_enum(name):
    for enum in _load_config().get("enums", []):
        if enum["name"] == name:
            return {v["value"]: v for v in enum["values"]}

    raise ValueError(f"no such enum! {name}")

def get_view(name):
    for view in _load_config().get("views", []):
        if view["name"] == name:
            return view

    raise ValueError(f"no such view! {name}")

def get_column(name):
    if name == "id":
        return {"field_name": "id", "display_name": "ID", "read_only": True}

    for col in _load_config().get("columns", []):
        if col["field_name"] == name:
            return col

    raise ValueError(f"no such column! {name}")

# Valid statuses & projects are read dynamically from the user's config.
STATUSES = get_enum("status")
PROJECTS = get_enum("projects")
