from peewee import fn
from peewee import Case, Value
from ..db import db, Thing
from .. import config


def get_thing(item_id: int) -> Thing:
    return Thing.get_by_id(item_id)


def add_thing(
    type: str = "?",
    **kwargs,
) -> Thing:
    """
    Add a new Thing to the database.
    Returns the created Thing instance.
    """
    with db.atomic():
        thing = Thing.create(
            type=type, data=kwargs
        )
    return thing


def update_thing(
    item_id: int,
    **kwargs,
) -> Thing:
    with db.atomic():
        thing = Thing.get_by_id(item_id)
        new_data = thing.data | kwargs
        query = Thing.update(data=new_data).where(Thing.id == item_id)
        query.execute()
        thing = Thing.get_by_id(item_id)
    return thing


def _parse_sort_string(sort_string):
    """
    Convert sort string like 'field1,-field2' to peewee order_by expressions.
    """
    sort_expressions = []

    if not sort_string:
        return sort_expressions

    for field_name in sort_string.split(","):
        is_desc = field_name.startswith("-")
        field_name = field_name[1:] if is_desc else field_name

        # TODO: look up field type
        field_type = "text"

        if field_type == "enum":
            # TODO: allow dynamic ordering
            order = config.get_enum(field_name)
            # CASE statement that maps each status to its position in the order
            order_case = Case(
                Thing.data[field_name],
                [(s, Value(i)) for i, s in enumerate(order)],
            )
            sort_expressions.append(order_case.desc() if is_desc else order_case.asc())
        elif field_type == "date":
            expr = fn.COALESCE(Thing.data[field_name], "3000-01-01")
            sort_expressions.append(expr.desc() if is_desc else expr)
        else:
            field_expr = Thing.data[field_name]
            sort_expressions.append(field_expr.desc() if is_desc else field_expr)

    return sort_expressions


def get_things(
    search_text: str | None = None,
    filters: dict[str, str] | None = None,
    sort: str = "",
) -> list[Thing]:
    query = Thing.select().where(~Thing.deleted)

    if search_text:
        # TODO: which fields are searchable should by dynamic
        query = query.where(fn.Lower(Thing.data['text']).contains(search_text.lower()))

    for param, val in filters.items():
        if val is not None:
            # no _in query for JSON fields, so use OR
            condition = False
            for cond in val:
                condition |= Thing.data[param] == cond
            query = query.where(condition)

    sort_expressions = _parse_sort_string(sort)
    query = query.order_by(*sort_expressions)

    return list(query)
