import datetime
from ..utils import (
    get_color_enum,
    get_colored_date,
)
from .modals import ChoiceModal, DateModal

class NotifyValidationError(Exception):
    """will notify and continue if raised"""


ELLIPSIS = "…"


class TableColumnConfig:
    def __init__(
        self,
        field: str,
        display_name: str,
        *,
        default=None,
        enable_editor=False,
        filterable=True,
        read_only=False,
    ):
        self.field = field
        self.display_name = display_name
        self.default = default
        self.enable_editor = enable_editor
        self.filterable = filterable
        self.read_only = read_only

    def preprocess(self, val):
        return val  # no-op

    def start_change(self, app, current_value):
        if current_value.endswith(ELLIPSIS):
            app.action_start_edit()
        else:
            # default edit mode
            app._show_input("edit", current_value)

    def format_for_display(self, val):
        val = str(val)
        if "\n" in val:
            val = val.split("\n")[0] + ELLIPSIS
        return val


class EnumColumnConfig(TableColumnConfig):
    def __init__(self, field: str, display_name: str, enum, **kwargs):
        super().__init__(field, display_name, **kwargs)
        self.enum = enum

    def preprocess(self, val):
        if val in self.enum:
            return val
        else:
            raise NotifyValidationError(f"Invalid value {val}. Use: {list(self.enum)}")

    def format_for_display(self, val):
        return get_color_enum(val, self.enum)

    def start_change(self, app, current_value):
        # a weird hack? pass app here and correct modal gets pushed
        app.push_screen(ChoiceModal(self.enum, current_value), app.apply_change)


class DateColumnConfig(TableColumnConfig):
    def preprocess(self, val):
        try:
            # ensure it parses, return as string
            datetime.datetime.strptime(val, "%Y-%m-%d")
            return val
        except ValueError:
            raise NotifyValidationError("Invalid date format. Use YYYY-MM-DD")

    def format_for_display(self, val):
        return get_colored_date(val)

    def start_change(self, app, current_value):
        app.push_screen(DateModal(current_value), app.apply_change)



def get_col_cls(field_type):
    return {
        "text": TableColumnConfig,
        "enum": EnumColumnConfig,
        "date": DateColumnConfig,
    }[field_type]
