import datetime
from textual.screen import ModalScreen
from textual.binding import Binding
from textual.widgets import Label
from textual.containers import Horizontal, Vertical
from textual.reactive import reactive
from ..utils import get_color_enum
from ..constants import SPECIAL_DATES_PIECES


class ConfirmModal(ModalScreen):
    CSS = """
    ConfirmModal {
        align: center middle;
        background: $primary 30%;
    }
    ConfirmModal Vertical {
        border: double teal;
        height: 4;
    }
    """

    BINDINGS = [
        ("y", "confirm", "Down"),
        ("n,escape", "cancel", "cancel"),
    ]

    def __init__(self, message):
        self.message = message
        super().__init__()

    def compose(self):
        with Vertical():
            yield Label(self.message)
            yield Label("[b](y)[/]es    [b](n)[/]o")

    def action_confirm(self):
        self.dismiss(True)

    def action_cancel(self):
        self.app.pop_screen()


class ChoiceModal(ModalScreen):
    CSS = """
    ChoiceModal {
        align: center middle;
        background: $primary 30%;
    }
    ChoiceModal Vertical {
        border: double teal;
        height: 10;
        width: 38;
    }
    ChoiceModal Label.hints {
        border: solid grey;
        height: 4;
    }
    ChoiceModal Label {
        height: 1;
    }
    ChoiceModal Label#selected {
        background: white;

    }
    """

    BINDINGS = [
        ("j,tab", "cursor_down", "Down"),
        ("k,shift+tab", "cursor_up", "Up"),
        Binding("enter", "select", "Select", priority=True),
        ("escape", "cancel", "cancel"),
    ]

    def __init__(self, enum, selected):
        self._enum = enum
        self.enum_by_idx = list(self._enum)
        # selection index
        self.sel_idx = 0
        # convert value back to index for initial selection
        for idx, e in enumerate(self._enum):
            if e.value == selected:
                self.sel_idx = idx
                break
        super().__init__()

    def compose(self):
        with Vertical():
            for idx, e in enumerate(self._enum):
                yield Label(
                    (f"> {idx} " if idx == self.sel_idx else f"  {idx} ")
                    + get_color_enum(e.value, self._enum),
                    classes="selected" if idx == self.sel_idx else "",
                )
            yield Label("""(h/j/k/l) move (0-9) quick select
(enter) confirm (esc) quit""", classes="hints")

    def _move_cursor(self, dir):
        labels = self.query(Label)
        # reset old
        labels[self.sel_idx].update(
            f"  {self.sel_idx} " + get_color_enum(self.enum_by_idx[self.sel_idx], self._enum)
        )
        # move cursor
        self.sel_idx = (self.sel_idx + dir) % len(self._enum)
        # reset new
        labels[self.sel_idx].update(
            f"> {self.sel_idx} " + get_color_enum(self.enum_by_idx[self.sel_idx], self._enum)
        )

    def action_cursor_down(self):
        self._move_cursor(1)

    def action_cursor_up(self):
        self._move_cursor(-1)

    async def action_select(self):
        self.dismiss(self.enum_by_idx[self.sel_idx])

    def action_cancel(self):
        self.app.pop_screen()

    def on_key(self, event) -> None:
        key = event.key
        if key in "0123456789":
            idx = int(key)
            self.dismiss(self.enum_by_idx[idx])


class DateModal(ModalScreen):
    CSS = """
    DateModal {
        align: center middle;
        background: $primary 30%;
    }
    DateModal Vertical {
        border: double teal;
        height: 10;
        width: 50;
    }
    DateModal Label {
        border: solid white;
        align: center middle;
    }
    DateModal Label.selected-date {
        border: solid green;
    }
    DateModal Label.hints {
        border: solid grey;
        height: 4;
    }
    """

    BINDINGS = [
        ("j", "cursor_down", "Down"),
        ("k", "cursor_up", "Up"),
        ("h,shift+tab", "cursor_left", "Left"),
        ("l,tab", "cursor_right", "Right"),
        ("f", "future", "Future"),
        ("t", "today", "Today"),
        ("u", "unclassified", "Unclassified"),
        Binding("enter", "select", "Select", priority=True),
        ("escape", "cancel", "cancel"),
    ]

    pieces = reactive([0, 0, 0], recompose=True)

    def __init__(self, date):
        super().__init__()
        if date in SPECIAL_DATES_PIECES:
            self.pieces = list(SPECIAL_DATES_PIECES[date])
        elif date:
            self.pieces = [int(p) for p in date.split("-")]
        else:
            self.action_today()
        self.selected = 1  # start on month

    def compose(self):
        with Vertical():
            with Horizontal():
                for i in range(3):
                    yield Label(
                        f"{self.pieces[i]}",
                        classes="selected-date" if self.selected == i else "",
                    )
            yield Label(
                """(h/j/k/l) move (enter) confirm (esc) quit
(p)ast (t)oday (f)uture""",
                classes="hints",
            )

    def action_future(self):
        self.pieces = list(SPECIAL_DATES_PIECES["future"])

    def action_unclassified(self):
        self.pieces = list(SPECIAL_DATES_PIECES["unclassified"])

    def action_today(self):
        today = datetime.date.today()
        self.pieces = [today.year, today.month, today.day]

    def action_cursor_left(self):
        # cycle Y/M/D
        self.selected = (self.selected - 1) % 3
        self.mutate_reactive(DateModal.pieces)

    def action_cursor_right(self):
        self.selected = (self.selected + 1) % 3
        self.mutate_reactive(DateModal.pieces)

    def max_for(self, piece):
        days_in_month = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
        if piece == 0:
            return 3000
        elif piece == 1:
            return 12
        else:
            # -1 for offset array
            return days_in_month[self.pieces[1] - 1]

    def _move_piece(self, by):
        cur_value = self.pieces[self.selected]
        cur_value += by
        if cur_value == 0:
            cur_value = self.max_for(self.selected)
        if cur_value > self.max_for(self.selected):
            cur_value = 1
        self.pieces[self.selected] = cur_value
        self.mutate_reactive(DateModal.pieces)

    def action_cursor_down(self):
        self._move_piece(-1)

    def action_cursor_up(self):
        self._move_piece(1)

    def on_key(self, event) -> None:
        key = event.key
        if key in "0123456789":
            cur_value = self.pieces[self.selected]
            # If we can append the values like 1+2 = 12
            # then do that, otherwise reset the value
            # so (for months) 1+3 = 3
            appended = int(str(cur_value) + key)
            if appended <= self.max_for(self.selected):
                cur_value = appended
            else:
                cur_value = int(key)
            self.pieces[self.selected] = cur_value
            self._move_piece(0)
            event.prevent_default()

    def action_select(self):
        self.dismiss("-".join(str(p) for p in self.pieces))

    def action_cancel(self):
        self.app.pop_screen()
