from textual.app import App, ComposeResult
from textual.containers import ScrollableContainer, Horizontal
from textual.widgets import DataTable, Static, Label
from textual.binding import Binding
from datetime import datetime

from ..controller.summaries import (
    get_category_summary,
    get_recently_active,
    get_due_soon,
)

####
#  Due Soon      # Task Summary
#                #
#                #
##################################
#  WIP           # Recently active
#                #
#                #
#                #


class CategoryTable(DataTable):
    """Table showing category summaries"""

    def on_mount(self):
        self.add_columns(
            "Category", "Zero", "WIP", "Blocked", "Done", "Overdue", "Due Soon"
        )
        self.refresh_data()

    def refresh_data(self):
        self.clear()
        summaries = get_category_summary(10)  # Show top 10 categories
        for summary in summaries:
            self.add_row(
                summary["category"],
                str(summary["tasks"]["zero"]),
                str(summary["tasks"]["wip"]),
                str(summary["tasks"]["blocked"]),
                str(summary["tasks"]["done"]),
                str(summary["tasks"]["overdue"]),
                str(summary["tasks"]["due_soon"]),
            )


def format_due_date(due_date: datetime | None) -> str:
    if not due_date:
        return "No due date"
    return due_date.strftime("%Y-%m-%d")


class DueTaskList(DataTable):
    """Table showing upcoming and overdue tasks"""

    def on_mount(self):
        self.add_columns("Task", "Category", "Due Date")
        self.refresh_data()

    def refresh_data(self):
        self.clear()
        tasks = get_due_soon(10, all_overdue=True)
        for task in tasks:
            self.add_row(
                task["text"],
                task["category"],
                format_due_date(task["due"]),
                key=str(task["id"]),
            )


class RecentTaskList(DataTable):
    """Table showing recently active tasks"""

    def on_mount(self):
        self.add_columns("Task", "Category", "Due Date")
        self.refresh_data()

    def refresh_data(self):
        self.clear()
        tasks = get_recently_active(10)
        for task in tasks:
            self.add_row(
                task["text"],
                task["category"] or "-",
                format_due_date(task["due"]),
                key=str(task["id"]),
            )


class Overview(App):
    """Task overview application"""

    CSS = """
    CategoryTable {
        height: 40%;
        margin: 1 1;
    }

    Label { 
      align: center middle; 
      background: purple;
    }
    
    #lists {
        height: 60%;
    }
    """

    BINDINGS = [
        Binding("q", "quit", "Quit"),
        Binding("r", "refresh", "Refresh Data"),
    ]

    def compose(self) -> ComposeResult:
        with Horizontal(id="top"):
            yield CategoryTable()
        with Horizontal(id="lists"):
            with ScrollableContainer():
                yield Label("Due Soon")
                yield DueTaskList()
            with ScrollableContainer():
                yield Label("Activity")
                yield RecentTaskList()

    def action_refresh(self):
        """Refresh all data in the tables"""
        self.query_one(CategoryTable).refresh_data()
        self.query_one(DueTaskList).refresh_data()
        self.query_one(RecentTaskList).refresh_data()


def run():
    app = Overview()
    app.run()
